class Config:
    def __init__(
        self,
        src_patterns=None,
        ignore_hidden_files=True,
        ignore_folders=None,
        without_hashes=False,
    ):
        if ignore_folders is None:
            ignore_folders = []

        if src_patterns is None:
            src_patterns = ["*.py"]

        self.ignore_folders = ignore_folders
        self.src_patterns = src_patterns
        self.ignore_hidden_files = ignore_hidden_files
        self.without_hashes = without_hashes



import argparse
import logging
import sys
from pathlib import Path

from lambda_packager.custom_log_formatter import CustomLogFormatter
from lambda_packager.package import LambdaAutoPackage


def parse_args(args):
    parser = argparse.ArgumentParser(
        description="Build code and dependencies into zip files that can be uploaded and run in AWS Lambda"
    )
    parser.add_argument(
        "--project-directory",
        dest="project_directory",
        required=False,
        default=None,
        help="""The path to the top level project directory.
                This is where source files and files that declare dependencies are expected to be held.
                Defaults to current directory""",
    )
    logging_default = logging.getLevelName(logging.INFO)
    parser.add_argument(
        "-l",
        "--log-level",
        dest="log_level",
        required=False,
        default=logging_default,
        help=f"set output verbosity, defaults to '{logging_default}'",
        choices=[
            logging.getLevelName(logging.DEBUG),
            logging.getLevelName(logging.INFO),
            logging.getLevelName(logging.WARNING),
            logging.getLevelName(logging.ERROR),
        ],
    )
    return parser.parse_args(args)


def run_cli():
    args = parse_args(sys.argv[1:])

    if args.project_directory:
        project_directory = Path(args.project_directory)
        assert project_directory.exists()
    else:
        project_directory = Path()

    log_level = logging.getLevelName(args.log_level)
    logger = logging.getLogger()
    logger.setLevel(log_level)

    ch = logging.StreamHandler()
    ch.setFormatter(CustomLogFormatter())
    ch.setLevel(log_level)
    logger.addHandler(ch)

    try:
        LambdaAutoPackage(logger=logger, project_directory=project_directory).execute()
    except Exception as e:
        logger.critical(e)
        raise e



import logging
import subprocess

import toml


class PoetryNotInstalled(Exception):
    pass


def poetry_is_used(current_directory):
    pyproject = current_directory.joinpath("pyproject.toml")
    if not pyproject.is_file():
        return False
    config = toml.loads(pyproject.read_text())

    try:
        requires = config["build-system"]["requires"][0].startswith("poetry")
        build_backend = config["build-system"]["build-backend"].startswith("poetry")

        return requires and build_backend
    except KeyError:
        return False


def export_poetry(target_path, project_directory=None, env=None, without_hashes=False):
    try:
        cmd = [
            "poetry",
            "export",
            "--no-interaction",
            "--verbose",
            "--with-credentials",
            "--format",
            "requirements.txt",
            "--output",
            target_path,
        ]

        if without_hashes:
            cmd.append("--without-hashes")

        subprocess.check_output(
            cmd,
            env=env,
            cwd=project_directory,
        )
    except FileNotFoundError as e:
        raise PoetryNotInstalled(
            "Please make sure you have poetry installed and in your path", e
        )
    except subprocess.CalledProcessError as e:
        logging.error(e.stdout.decode())
        raise e



import logging
import shutil
import tempfile
from pathlib import Path

import toml

from lambda_packager.config import Config
from lambda_packager.handle_poetry import poetry_is_used, export_poetry
from lambda_packager.handle_requirements_txt import install_requirements_txt


class NoSrcFilesFound(Exception):
    pass


class LambdaAutoPackage:
    def __init__(self, config=None, project_directory=None, logger=None):
        if project_directory:
            self.project_directory = Path(project_directory)
        else:
            self.project_directory = Path()

        if logger:
            self.logger = logger
        else:
            self.logger = logging.getLogger(__name__)
            self.logger.debug("set up self logging")

        if config:
            self.config = config
        else:
            self.config = LambdaAutoPackage._get_config(
                self.project_directory.joinpath("pyproject.toml")
            )

        self.tmp_folder = self._create_tmp_directory()

    def execute(self):

        if self.project_directory.joinpath("requirements.txt").is_file():
            self.logger.info("using requirements.txt file in project directory")
            requirements_file_path = self.project_directory.joinpath("requirements.txt")
            install_requirements_txt(
                str(self.tmp_folder), requirements_file_path=requirements_file_path
            )
        elif poetry_is_used(self.project_directory):
            self.logger.info("using pyproject.toml file in project directory")
            requirements_file_path = self.tmp_folder.joinpath("requirements.txt")
            export_poetry(
                target_path=requirements_file_path,
                project_directory=self.project_directory,
                without_hashes=self.config.without_hashes,
            )
            install_requirements_txt(
                str(self.tmp_folder),
                requirements_file_path=requirements_file_path,
                no_deps=True,
            )
        else:
            self.logger.warning("No dependency found, none will be packaged")

        self._copy_source_files(
            source_dir=self.project_directory,
            target_dir=self.tmp_folder,
        )

        self._create_zip_file(
            self.tmp_folder, str(self.project_directory.joinpath("dist/lambda.zip"))
        )

    @staticmethod
    def _create_tmp_directory():
        dirpath = tempfile.mkdtemp()
        test_path = Path(dirpath)
        assert test_path.exists()
        assert test_path.is_dir()
        return test_path

    def _copy_source_files(self, source_dir: Path, target_dir: Path):
        matching_objects = LambdaAutoPackage._get_matching_files_and_folders(
            self.config.src_patterns, source_dir
        )

        self.logger.info(f"copying {len(matching_objects)} matching_objects")
        self.logger.debug(f"copying {matching_objects} matching_objects")

        copied_locations = []
        for src in matching_objects:
            relative_path = src.relative_to(source_dir)
            new_location = target_dir.joinpath(relative_path)

            if self._is_ignored_file(src.resolve()):
                self.logger.warning(f"skipping path {src.resolve()}")
            elif src.is_file():
                self.copy_file(src, new_location, copied_locations)
            elif src.is_dir():
                self.copy_directory(src, new_location, copied_locations)
            else:
                self.logger.warning(f"the path '{src}' was nether a file or directory")

        if len(copied_locations) <= 0:
            raise NoSrcFilesFound(
                "No src files were found. This is likely a problem. Exiting now to highlight this"
            )

        copied_locations_string = "\n".join(copied_locations)
        self.logger.info(f"copied the following locations: \n{copied_locations_string}")

    def copy_file(self, src, new_location, copied_locations):
        self.logger.debug(f"about to copy file from {src} --> {new_location}")
        new_location.parent.mkdir(exist_ok=True, parents=True)
        copied_locations.append(str(shutil.copyfile(src, new_location)))

    def copy_directory(self, src, new_location, copied_locations):
        self.logger.debug(f"about to copy directory from {src} --> {new_location}")
        copied_locations.append(
            str(
                shutil.copytree(
                    src=str(src),
                    dst=str(new_location),
                    dirs_exist_ok=True,
                    ignore=(
                        self._is_ignored_file_list
                        if self.config.ignore_hidden_files or self.config.ignore_folders
                        else None
                    ),
                )
            )
        )

    def _is_ignored_file_list(self, src, files):
        if self._is_ignored_file(Path(src).resolve()):
            self.logger.warning(f"skipping folder {Path(src).resolve()}")
            return files

        files_to_skip = {}
        for file in files:
            path = Path(file).resolve()
            if self._is_ignored_file(path):
                files_to_skip[file] = path

        if files_to_skip:
            self.logger.warning(f"skipping path {list(files_to_skip.values())}")
        return files_to_skip.keys()

    def _is_ignored_file(self, resolved_path: Path):
        path = str(resolved_path)

        if self.config.ignore_hidden_files:
            if path.startswith(".") or "/." in path:
                return True

        for folder in self.config.ignore_folders:
            if f"/{folder}" in path:
                return True

        return False

    @staticmethod
    def _get_matching_files_and_folders(pattern_list, source_dir):
        matching_objects = set()
        for pattern in pattern_list:
            matching_objects.update(source_dir.rglob(pattern))

        return matching_objects

    @staticmethod
    def _create_zip_file(source_dir, target):
        if target.endswith(".zip"):
            Path(target).parent.mkdir(exist_ok=True)
            name = target[: -len(".zip")]
            shutil.make_archive(base_name=name, format="zip", root_dir=source_dir)
        else:
            raise ValueError(
                f"given target path '{target}' does not end with correct extension. should end with '.zip'"
            )

    @staticmethod
    def _read_config(file: Path):
        config = toml.loads(file.read_text())

        try:
            return config["tool"]["lambda-packager"]
        except KeyError:
            logging.warning("no config found!")
            return {}

    @staticmethod
    def _get_config(file: Path):
        if not file.is_file():
            logging.warning("no config file found!")
            return Config()

        config_dict = LambdaAutoPackage._read_config(file)
        return Config(**config_dict)



import logging
import subprocess
import sys
from pathlib import Path


def install_requirements_txt(target, requirements_file_path: Path, no_deps=False):
    # https://pip.pypa.io/en/stable/user_guide/#using-pip-from-your-program
    if not requirements_file_path.is_file():
        raise ValueError(
            f"could not find requirements.txt file at '{requirements_file_path}'"
        )

    logging.info(f"installing pip requirements to '{target}'")
    cmd = [
        sys.executable,
        "-m",
        "pip",
        "install",
        "-r",
        requirements_file_path,
        "--target",
        target,
    ]
    if no_deps:
        cmd.append("--no-deps")

    output = subprocess.check_output(cmd)
    logging.debug(output.decode())
    return output



# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['lambda_packager']

package_data = \
{'': ['*']}

install_requires = \
['toml>=0.10.2,<0.11.0']

entry_points = \
{'console_scripts': ['lambda-packager = lambda_packager:run_cli']}

setup_kwargs = {
    'name': 'lambda-packager',
    'version': '1.2.0',
    'description': 'Stop writing your own scripts and let this package your python aws lambda zips for you',
    'long_description': '# lambda-packager\n\n<a href="https://github.com/hmrc"><img alt="HMRC: Digital" src="https://img.shields.io/badge/HMRC-Digital-FFA500?style=flat&labelColor=000000&logo=gov.uk"></a>\n<a href="https://pypi.org/project/lambda-packager/"><img alt="PyPI" src="https://img.shields.io/pypi/v/lambda-packager"></a>\n<a href="https://pypi.org/project/lambda-packager/"><img alt="Python" src="https://img.shields.io/pypi/pyversions/lambda-packager"></a>\n<a href="https://github.com/hmrc/python-aws-lambda-packager/blob/master/LICENSE"><img alt="License: Apache 2.0" src="https://img.shields.io/github/license/hmrc/python-aws-lambda-packager"></a>\n<a href="https://github.com/psf/black"><img alt="Code style: black" src="https://img.shields.io/badge/code%20style-black-000000.svg"></a>\n\nThe purpose of this tool is to avoid writing AWS lambda packaging scripts repeatedly. It is intended to give a consistent output regardless of how you currently define your python dependencies. The tool was built as most existing tools are built into larger frameworks that have other considerations when adopting\n\nCurrently, requires python >=3.8 and later due to [required features of copytree](https://docs.python.org/3/library/shutil.html#shutil.copytree)\n\n## Usage\n- You can run with the following:\n```bash\n$ lambda-packager\n # or if not in the project directory  \n$ lambda-packager --project-directory path/to/project/dir\n```\n- lambda-packager will include any dependencies defined in\n    - poetry (pyproject.toml)\n    - requirements.txt\n    - ~~Pipenv~~ (Coming soon!)\n- By default lambda-packager will include all src files that match `*.py`\n- You can customise this through config in `pyproject.toml`:\n```toml\n[tool.lambda-packager]\nsrc_patterns = ["lambda_packager/*.py"]\n```\n\n### Hidden files\n- Hidden files and folders are ignored by default when including src files\n- if you wish to disable this, then add the following config to your `pyproject.toml`\n```toml\n[tool.lambda-packager]\nignore_hidden_files = false\n```\n\n### Ignore folders\nIf there are folders that you wish always exclude, then you can use `ignore_folders`\nNote: `ignore_folders` is always respected even if there was a match via `src_patterns`\n```toml\n[tool.lambda-packager]\nignore_folders = ["venv"]\n```\n\n### Ignore hashes\nOnly has an effect when using poetry `pyproject.toml` files\n\nSkips exporting hashes from poetry to avoid issues when using non-pypi packages \nby providing `--without-hashes` flag when calling `poetry export`\nSee https://github.com/hmrc/python-aws-lambda-packager/issues/2 for more info (Note: version number remains pinned when this is enabled)\n```toml\nwithout_hashes = True\n```\n\n### Full usage\n```\nusage: lambda-packager [-h] [--project-directory PROJECT_DIRECTORY] [-l {DEBUG,INFO,WARNING,ERROR}]\n\nBuild code and dependencies into zip files that can be uploaded and run in AWS Lambda\n\noptional arguments:\n  -h, --help            show this help message and exit\n  --project-directory PROJECT_DIRECTORY\n                        The path to the top level project directory. This is where source files and files that declare dependencies are expected to be held. Defaults to current directory\n  -l {DEBUG,INFO,WARNING,ERROR}, --log-level {DEBUG,INFO,WARNING,ERROR}\n                        set output verbosity, defaults to \'INFO\'\n\n```\n\n## License\n\nThis code is open source software licensed under the [Apache 2.0 License]("http://www.apache.org/licenses/LICENSE-2.0.html").\n',
    'author': 'cob16',
    'author_email': 'public+github@cormacbrady.info',
    'maintainer': None,
    'maintainer_email': None,
    'url': 'https://github.com/hmrc/python-aws-lambda-packager',
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'entry_points': entry_points,
    'python_requires': '>=3.8.0,<4.0.0',
}


setup(**setup_kwargs)
